{
  This demo shows how to add a table to imitate 1/2 columns mode
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, StdCtrls, RVScroll, RichView, RVEdit, CRVData, RVTable;

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    Button1: TButton;
    Button2: TButton;
    RVStyle1: TRVStyle;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function GetColCount: Integer;
    function GetRVFStreamWithoutStyles(RVData: TCustomRVData): TMemoryStream;
    procedure To1Column;
    procedure To2Columns;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

// Returns the current count of columns, or 0 if no columns
function TForm1.GetColCount: Integer;
var table: TRVTableItemInfo;
begin
  Result := 0;
  if (RichViewEdit1.ItemCount<>1) or (RichViewEdit1.GetItemStyle(0)<>rvsTable) then
    exit;
  table := TRVTableItemInfo(RichViewEdit1.GetItem(0));
  if table.RowCount<>1 then
    exit;
  Result := table.ColCount;
end;

// Checking if the cell has a default (empty) content
// (1 empty text line of 0 style)
function IsEmpty(RVData: TCustomRVData): Boolean;
begin
  Result := (RVData.ItemCount=1) and (RVData.GetItemStyle(0)=0) and
    (RVData.GetItemPara(0)=0) and (RVData.GetItemText(0)='')
end;

// Returns RVF content in a stream
// This RVF does not contain collections of styles
// (important, because Cell.LoadRVFFromStream does not support styles!)
function TForm1.GetRVFStreamWithoutStyles(RVData: TCustomRVData): TMemoryStream;
begin
  Result := TMemoryStream.Create;
  RichViewEdit1.RVFOptions := RichViewEdit1.RVFOptions-[rvfoSaveTextStyles, rvfoSaveParaStyles];
  RVData.SaveRVFToStream(Result, False, clNone, nil, nil);
  RichViewEdit1.RVFOptions := RichViewEdit1.RVFOptions+[rvfoSaveTextStyles, rvfoSaveParaStyles];
  Result.Position := 0;
end;

procedure TForm1.To1Column;
var ColCount: Integer;
    table: TRVTableItemInfo;
    Stream: TMemoryStream;
    Color: TColor;
begin
  ColCount := GetColCount;
  if ColCount=1 then
    exit; // already 1 column
  if (ColCount<>2) then begin
    // copying the whole content into a table
    table := TRVTableItemInfo.CreateEx(1, 1, RichViewEdit1.RVData);
    Stream := GetRVFStreamWithoutStyles(RichViewEdit1.RVData);
    table.Cells[0,0].LoadRVFFromStream(Stream, Color, nil, nil);
    Stream.Free;
    RichViewEdit1.Clear;
    RichViewEdit1.AddItem('', table);
    end
  else begin
    RichViewEdit1.SetSelectionBounds(0,0,0,0);
    table := TRVTableItemInfo(RichViewEdit1.GetItem(0));
    if not IsEmpty(table.Cells[0,1].GetRVData) then begin
      // copying the second column to the end of the first one
      table.Cells[0,0].AddNL('',0,0);
      Stream := GetRVFStreamWithoutStyles(table.Cells[0,1]);
      table.Cells[0,0].InsertRVFFromStream(Stream, table.Cells[0,0].ItemCount,
        Color, nil, nil, False);
      Stream.Free;
    end;
    // deleting the second column
    table.DeleteCols(1,1,False);
    RichViewEdit1.ClearUndo;
  end;
  table.BestWidth := 0;
  table.Cells[0,0].BestWidth := 345*2;
  RichViewEdit1.Format;
end;

procedure TForm1.To2Columns;
var ColCount: Integer;
    table: TRVTableItemInfo;
    Stream: TMemoryStream;
    Color: TColor;
begin
  ColCount := GetColCount;
  if ColCount=2 then
    exit; // already 2 columns
  if (ColCount<>1) then begin
    // copying the whole content into a table
    table := TRVTableItemInfo.CreateEx(1, 2, RichViewEdit1.RVData);
    Stream := GetRVFStreamWithoutStyles(RichViewEdit1.RVData);
    table.Cells[0,0].LoadRVFFromStream(Stream, Color, nil, nil);
    Stream.Free;
    RichViewEdit1.Clear;
    RichViewEdit1.AddItem('', table);
    end
  else begin
    // adding a new empty column
    table := TRVTableItemInfo(RichViewEdit1.GetItem(0));
    RichViewEdit1.SetSelectionBounds(0,0,0,0);
    table.InsertCols(1,1,-1,False);
    RichViewEdit1.ClearUndo;
  end;
  table.BestWidth := 0;
  table.Cells[0,0].BestWidth := 345;
  table.Cells[0,1].BestWidth := 345;  
  RichViewEdit1.Format;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  To1Column;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  To2Columns;
end;

end.
